package book.wsad.channel.ejbclient;

import java.util.*;
import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import book.wsad.channel.ejb.*;
import book.wsad.channel.bean.BookInfo;

public class RequireParts {
	
	private LinkPartsHome linkPartsHome;
    private DocumentBuilder builder;

	/**
	 * Constructor for RequireParts
	 */
	public RequireParts() {
		super();
	}

	private DocumentBuilder getDocumentBuilder() throws Exception {
		if (builder == null) {
			try {
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				builder = factory.newDocumentBuilder();
			} catch (ParserConfigurationException pce) {
				pce.printStackTrace();
				throw new Exception("Error creating document builder:" + pce.getMessage());
			}
		}
		return builder;
	}

	private LinkPartsHome getLinkPartsHome() throws Exception {
		if (linkPartsHome == null) {
			try {
				InitialContext ic = new InitialContext();
				Object objref = ic.lookup("book/wsad/LinkParts");
				linkPartsHome =
					(LinkPartsHome) PortableRemoteObject.narrow(objref, LinkPartsHome.class);
			} catch (NamingException ne) {
				ne.printStackTrace();
				throw new Exception("Error looking up LinkPartsHome:" + ne.getMessage());
			}
		}
		return linkPartsHome;
	}

	private Element newElement(Document doc, String name, String value) {
		Element element = doc.createElement(name);
		element.appendChild(doc.createTextNode(value));
		return element;
	}

	private Element createPart(Document doc, BookInfo bean) throws DOMException {
		Element elPart = doc.createElement("Part");
		elPart.appendChild(newElement(doc, "PartName", bean.getPartName()));
		elPart.appendChild(
			newElement(doc, "ItemNubmer", bean.getItemNumber().toString()));
		elPart.appendChild(newElement(doc, "Title", bean.getTitle()));
		elPart.appendChild(newElement(doc, "Author", bean.getAuthor()));
		elPart.appendChild(newElement(doc, "Publisher", bean.getPublisher()));
		elPart.appendChild(
			newElement(doc, "PublishingDate", bean.getPublishingDate().toString()));
		elPart.appendChild(
			newElement(
				doc,
				"NetPrice",
				(new Integer(bean.getNetPrice().toString())).toString()));
		;
		elPart.appendChild(
			newElement(
				doc,
				"ReducePrice",
				(new Integer(bean.getReducePrice().toString())).toString()));
		elPart.appendChild(
			newElement(
				doc,
				"DiscountRate",
				(new Integer(bean.getDiscountRate().toString())).toString()));
		;
		elPart.appendChild(
			newElement(
				doc,
				"Quantity",
				(new Integer(bean.getQuantity().toString())).toString()));
		return elPart;
	}

	public Element requireParts(String partCode) throws Exception {
		Element result = null;
		Document doc = null;
		DocumentBuilder builder;
		LinkParts linkParts;
		doc = getDocumentBuilder().newDocument();
		result = doc.createElement("RequirePartsResult");
		result.setAttribute(
			"xmlns",
			"http://www.kinet.co.kr/WSAD/schemas/RequireResults");
		result.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		result.setAttribute(
			"xsi:schemaLocation",
			"http://www.kinet.co.kr/WSAD/schemas/RequireResults wsdl/RequireParts.xsd");
		try {
			linkParts = getLinkPartsHome().create();
			Vector resultData = linkParts.linkParts(partCode);
			int len = resultData.size();
			for (int i = 0; i < len; i++) {
				result.appendChild(createPart(doc, (BookInfo) resultData.elementAt(i)));
			}
		} catch (CreateException ce) {
			ce.printStackTrace();
			throw new Exception("Error finding part:" + ce.getMessage());
		} catch (java.rmi.RemoteException re) {
			re.printStackTrace();
			throw new Exception("Error with session EJB: " + re.getMessage());
		}
		return result;
	}
}